\begin{verbatim}
C ==================================================================
C                           Programm OS_WHEAT
C
C    Autor:  Juergen Berlekamp
C            Institut fuer Umweltsystemforschung
C            der Universitaet Osnabrueck
C
C    Dynamisches Simulationsmodell des Wachstums von Winterweizen-
C    bestaenden. Modelliert werden die Prozesse Photosynthese,
C    Respiration, Konversion Glc<->Assimilate, Allokation und
C    Translokation fuer die Kompartimente Blaetter, Halme, Wurzeln,
C    Aehren und Endosperm. Der Bestand ist in bis zu 11 horizontale
C    Bestandesschichten differenziert. Treibende Kraefte des Modells
C    sind die Umgebungstemperatur und die photosynthetisch aktive
C    Strahlung.
C
C    Das Modell wird als System von 75 gekoppelten gewoehnlichen
C    Differentialgleichungen behandelt. Die Integration erfolgt
C    mit Hilfe der Runge-Kutta-Methode.
C
C    Das vorliegende FORTRAN-77-Programm ist mit kleineren
C    Aenderungen (Input-Output) auf zahlreichen Rechnern lauf-
C    faehig. Einzige Voraussetzung ist die Verfuegbarkeit der
C    NAG-Routine D02BBF zur Integration.
C
C ==================================================================
C MAIN Programm
C -------------------------------------------------------------------
C Variablen:
C
C
C B0(j)    Phytomasse des i-ten Pools zum Zeitpunkt T0      [gC/m^2]
C          der Index j laesst sich wie folgt bestimmen:
C          j = (l-1)*6 + 1  Struktur   Blatt der Schicht l
C          j = (l-1)*6 + 2     "       Halm      "
C          j = (l-1)*6 + 3  Assimilate Blatt     "
C          j = (l-1)*6 + 4     "       Halm      "
C          j = (l-1)*6 + 5  Glucose    Blatt     "
C          j = (l-1)*6 + 6     "       Halm      "
C          j = NLAYER*6+ 1  Struktur   Wurzel
C          j = NLAYER*6+ 2     "       Aehre
C          j = NLAYER*6+ 3  Assimilate Wurzel
C          j = NLAYER*6+ 4     "       Aehre
C          j = NLAYER*6+ 5  Glucose    Wurzel
C          j = NLAYER*6+ 6     "       Aehre
C          j = NLAYER*6+ 7  Endosperm
C          j = NLAYER*6+ 8  Atmosphaere     (Hilfspool)
C          J = NLAYER*6+ 9  Bestandesabfall (Hilfspool)
C
C Y(j)      Phytomasse des j-ten Pools zum Zeitpunkt T      [gC/m^2]
C X0,TANFG  Simulationsbeginn                               [d]
C X1,TEND   Simulationsende                                 [d]
C YLIMIT(i) maximale Phytomasse pro Schicht fuer
C           Blaetter (i=1) und Halme (i=2)
C BIO(i,l)  Phytomasse v. Blaettern (i=1) und Halmen (i=2)
C           der Schicht l
C KPH(i)    Quantenwirksamkeit der Photosynthese            [gC/Mol]
C KR(i)     Respirationskoeffizient                         [1/d]
C KE(i)     Extinktionskoeffizient                          [m^2/g]
C KL(i)     Koeffizient der Bestandesabfallproduktion       [1/d]
C KA(i)     Allokationskoeffizient                          [1/d]
C KT        Transloationskoeffizient                        [1/d]
C KG1       Koeffizient d. Konversion Glucose->Assimilate   [1/d]
C KG2       Koeffizient d. Konversion Assimilate->Glucose   [1/d]
C TNEXT1    Zeitpunkt der naechsten Bildschirmausgabe       [d]
C TNEXT2    Zeitpunkt der naechsten Dateiausgabe            [d]
C TSCRE     Differenz zweier Bildschirmausgaben             [d]
C TFILE     Differenz zweier Dateiausgaben                  [d]
C TOL       Fehlertoleranz
C W(j,10)   Hilfsfeld
C
C KAPTS     Dateiname der  KA-Tabellenwerte
C KPHPTS    Dateiname der KPH-Tabellenwerte
C ENVIN     Dateiname der PAR- und Temp.werte
C POUT1..3  Dateiname der Poolgroessen
C FOUT1..2  Dateiname der Flussgroessen
C INI       Name der Initialisierungsdatei
C
C IRELAB    Fehlersteuerung
C IFAIL     Fehlermeldung
C N         Anzahl der DGLs
C NLAYERS   Zahl der Bestandesschichten
C MAXLAY(i) Zahl der gefuellten Blatt- (i=1) und Halmschichten (i=2)
C ML        hoechste gefuellte Schicht
C i,j       Hilfsvariablen
C
C -------------------------------------------------------------------
      REAL*8  B0(75), Y(75), X0, X1, YLIMIT(2), BIO(2,11)
      REAL*8  KPH(4),KR(4),KE(4),KL(4),KA(4),KT,KG1,KG2
      REAL*8  TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      REAL*8  TOL, W(75,10)
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
      INTEGER IRELAB, IFAIL, N, NLAYERS, MAXLAY(2), ML, i,j
C
      COMMON /BPOOLS/ B0
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
      COMMON /KOEFF/  KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /GEN1/   YLIMIT, NLAYERS
      COMMON /LAYER/  MAXLAY
      COMMON /ZEIT/   TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
C
      EXTERNAL FCN, OUTPUT
C
      SAVE
C
      INI  = 'OS-WHEAT.INI'
C
      TOL    = 1.E-7
      IRELAB = 0
      IFAIL  = 0
C
      MAXLAY(1) = 1
      MAXLAY(2) = 1
C
      CALL RdIni()
      CALL RdEnv()
      CALL RdKA()
      CALL RdKPH()
      CALL IniOut()
C
      DO 10 i=1,75
        Y(i) = B0(i)
 10   CONTINUE
C
      N = NLAYERS * 6 + 9
C
      X1 = TANF
C
 20   X0 = X1
      X1 = X1 + 1.0
C
C     -------------------------------------------------
C     Phytomasse berechnen und hoechste gefuellte Schicht
C     ermitteln
C     -------------------------------------------------
      MAXLAY(1)=1
      MAXLAY(2)=1
      DO 30 i=1,11
        BIO(1,i) = Y(i*6-6+1) + Y(i*6-6+3) + Y(i*6-6+5)
        BIO(2,i) = Y(i*6-6+2) + Y(i*6-6+4) + Y(i*6-6+6)
        IF ( BIO(1,i) .GT. 0.0001 ) MAXLAY(1) = i
        IF ( BIO(2,i) .GT. 0.0001 ) MAXLAY(2) = i
 30   CONTINUE
      IF (MAXLAY(1) .NE. MAXLAY(2)) THEN
        ML = MAX(MAXLAY(1), MAXLAY(2))
      ELSE
        ML = MAXLAY(1)
      ENDIF
C
C     -------------------------------------------------
C     Erstes Fuellen der Aehren, wenn Aehren-KA groesser
C     0.0001 und bisher keine Aehrenstruktur vorhanden
C     -------------------------------------------------
C
      CALL ActKoe(X0)
      IF ( (KA(4) .GT. 0.0001) .AND.
     &     (Y(NLAYERS*6+2) .LT. 0.0001) ) THEN
C       -----------------------------------------------
C       Fuellung der Aehrenpools (Struktur, Assimilate,
C       Glucose) mit 10% der obersten Halme
C       -----------------------------------------------
        Y(NLAYERS*6+2) = Y(NLAYERS*6+2) + Y(ML*6-6+2) * 0.1
        Y(ML*6-6+2)    = Y(ML*6-6+2)    - Y(ML*6-6+2) * 0.1
        Y(NLAYERS*6+4) = Y(NLAYERS*6+4) + Y(ML*6-6+4) * 0.1
        Y(ML*6-6+4)    = Y(ML*6-6+4)    - Y(ML*6-6+4) * 0.1
        Y(NLAYERS*6+6) = Y(NLAYERS*6+6) + Y(ML*6-6+6) * 0.1
        Y(ML*6-6+6)    = Y(ML*6-6+6)    - Y(ML*6-6+6) * 0.1
      ENDIF
C
C     -------------------------------------------------
C     Integration von T=X0 bis T=X1 (ein Tag)
C     -------------------------------------------------
C
      CALL D02BBF (X0,X1,N,Y,TOL,IRELAB,FCN,OUTPUT,W,IFAIL)
C
C     -------------------------------------------------
C     Fuellen einer neuen Schicht, wenn Phytomasse der
C     obersten Schicht bei Blaettern oder Halmen 25%
C     des Schwellenwertes erreicht hat.
C     -------------------------------------------------
      IF( ((BIO(1,MAXLAY(1)) .GT. YLIMIT(1) * 0.25)    .OR.
     &     (BIO(2,MAXLAY(2)) .GT. YLIMIT(2) * 0.25)) ) THEN
        DO 31 i=1,6
          Y(ML*6+i)   = Y(ML*6+i)
     &                + Y(ML*6-6+i) * 0.2
          Y(ML*6-6+i) = Y(ML*6-6+i)
     &                - Y(ML*6-6+i) * 0.2
 31     CONTINUE
      ENDIF
C
C     -------------------------------------------------
C     Fortfuehrung der Integration, wenn Endzeit bisher
C     nicht erreicht ist
C     -------------------------------------------------
      IF (X1 .LT. TEND) GOTO 20
C
C     -------------------------------------------------
C     Ende der Ausgabe
C     -------------------------------------------------
      CALL EndOut()
C
      STOP
      END
C ====================================================================
      SUBROUTINE RdIni()
C     -------------------------------------------------
C     Subroutine RdIni: Einlesen der Startwerte aus der
C     Initialisierungsdatei
C     -------------------------------------------------
C
      REAL*8       B0(75), YLIMIT(2)
      REAL*8       TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      REAL*8       KPH(4),KE(4),KR(4),KL(4),KA(4),KT,KG1,KG2
      REAL*8       DPAR, DTEMP
      INTEGER      i,j, NLAYERS
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      COMMON /ZEIT/   TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      COMMON /GEN1/   YLIMIT, NLAYERS
      COMMON /KOEFF/  KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /BPOOLS/ B0
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
      COMMON /SZENAR/ DTEMP, DPAR
C
      SAVE
C
C     -------------------------------------------------
C     Einlesen der Dateinamen
C     -------------------------------------------------
      OPEN(10,FILE=INI)
C
      READ(10,90) CDUM, KAPTS
      READ(10,90) CDUM, KPHPTS
      READ(10,90) CDUM, ENVIN
C
      READ(10,90) CDUM, POUT1
      READ(10,90) CDUM, POUT2
      READ(10,90) CDUM, POUT3
C
      READ(10,90) CDUM, FOUT1
      READ(10,90) CDUM, FOUT2
C     -------------------------------------------------
C     Einlesen der Schichtenzahl, der Anfangs- und End-
C     zeit, und der Phytomassengrenzen pro Schicht bei
C     Blaettern und Halmen
C     -------------------------------------------------
      READ(10,91) CDUM, NLAYERS
C
      READ(10,92) CDUM, TANF
      READ(10,92) CDUM, TEND
C
      READ(10,92) CDUM, YLIMIT(1)
      READ(10,92) CDUM, YLIMIT(2)
C     -------------------------------------------------
C     Einlesen der Zeitintervalle fuer Bildschirm- und
C     Dateiausgabe
C     -------------------------------------------------
      READ(10,92) CDUM, TSCRE
      READ(10,92) CDUM, TFILE
C     -------------------------------------------------
C     Einlesen des Translokationskoeffizienten
C     -------------------------------------------------
      READ(10,92) CDUM, KT
C     -------------------------------------------------
C     Einlesen der Konversionskoeffizienten
C     -------------------------------------------------
      READ(10,92) CDUM, KG1
      READ(10,92) CDUM, KG2
C     -------------------------------------------------
C     Einlesen linearer Temperatur- und PAR-Aenderungen
C     (nur fuer Szenarien benoetigt)
C     -------------------------------------------------
      READ(10,92) CDUM, DTEMP
      READ(10,92) CDUM, DPAR
C     -------------------------------------------------
C     Einlesen der Startwerte der Pools
C     -------------------------------------------------
      READ(10,'(1A40)') CDUM
      DO 10 i=1,75
        B0(i) = 0.0
10    CONTINUE
      READ(10,*) B0(NLAYERS*6+1), B0(NLAYERS*6+3), B0(NLAYERS*6+5)
      DO 11 i = 0,11-1
        READ(10,*) B0(i*6+1),B0(i*6+3),B0(i*6+5),
     &             B0(i*6+2),B0(i*6+4),B0(i*6+6)
11    CONTINUE
      READ(10,*) B0(NLAYERS*6+2), B0(NLAYERS*6+4), B0(NLAYERS*6+6),
     &           B0(NLAYERS*6+7)
C
C     -------------------------------------------------
C     Startwerte fuer die Hilfspools Atmosphaere und
C     Bestandesabfall setzten
C     -------------------------------------------------
      B0(NLAYERS*6+8) = 10000.0
      B0(NLAYERS*6+9) = 0.0
C
      CLOSE(10)
C
  90  FORMAT(1A40, 1A40)
  91  FORMAT(1A40, 1I5)
  92  FORMAT(1A40, 1E15.0)
C
      RETURN
      END
C ====================================================================
      SUBROUTINE RdKA()
C     -------------------------------------------------
C     Subroutine RdKA: Einlesen der Allokationskoeffi-
C     zienten (Tabellenwerte) und Speicherung in der
C     Var. KAW(j,i) mit j=Kompartiment u. i=Kalendertag
C     -------------------------------------------------
      REAL*8 KAW(4,365), xzeit, y(4)
      INTEGER NVALUE, i, j, izeit
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      COMMON /KAWERT/ KAW
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      SAVE /KAWERT/,/FILES/
C
      DO 20 i=1,365
        DO 22 j=1,4
          KAW(j,i) = -99.0
 22     CONTINUE
 20   CONTINUE
C
      OPEN(20,FILE=KAPTS,STATUS='OLD')
C
1     READ(20,*,END=50,ERR=99) xzeit, (y(i),i=1,4)
      izeit = INT(xzeit+0.5)
      DO 10 i=1,4
        KAW(i,izeit) = y(i)
 10   CONTINUE
      GOTO 1
C
 99   WRITE(*,*)' RdKA: Einlesefehler'
      CLOSE(20)
      STOP
C
 50   CLOSE(20)
      RETURN
      END
C ====================================================================
      SUBROUTINE RdKPH()
C     -------------------------------------------------
C     Subroutine RdKPH: Einlesen der Tabellenwerte der
C     Quantenwirksamkeiten und Speicherung in dem Feld
C     KPHW(i) mit i=INT(PAR/10.)
C     -------------------------------------------------
      REAL*8 KPHW(200) ,xpar, x
      INTEGER i,ipar
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      COMMON /KPHWERT/ KPHW
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      SAVE /KPHWERT/, /FILES/
C
      DO 1 i=0,200
        KPHW(i) = 0.0
   1  CONTINUE
C
      OPEN(22,FILE=KPHPTS,STATUS='OLD')
C
  10  READ(22,*,END=50,ERR=99) xpar, x
      ipar       = INT(xpar)/10
      KPHW(ipar) = x
      GOTO 10
C
  99  WRITE(*,*) 'RdKPH : Einlesefehler'
      CLOSE(22)
      STOP
C
  50  CLOSE(22)
C
      RETURN
      END
C ====================================================================
      SUBROUTINE RdEnv()
C     -------------------------------------------------
C     Subroutine RdEnv: Einlesen der Wetterdaten
C     Temperatur und PAR. Speicherung in den Feldern
C     TEMPAR(i) und PARARR(i) mit i=T*48+0.5
C     -------------------------------------------------
      REAL*8 PARARR(30000), TEMPAR(30000), T, PAR, TEMP
      REAL*8 DTEMP, DPAR
      INTEGER i, INDEX
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      COMMON /READ/   PARARR, TEMPAR
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
      COMMON /SZENAR/ DTEMP, DPAR
C
      SAVE /READ/, /FILES/,/SZENAR/
C
      DO 10 i=1,30000
        TEMPAR(i) = -99.0
        PARARR(i) = -99.0
 10   CONTINUE
C
      OPEN(25,FILE=ENVIN,STATUS='OLD')
1     READ(25,*,END=20,ERR=99) T, TEMP, PAR
      INDEX         = INT(T * 4. * 24. + 0.5)
      TEMPAR(INDEX) = TEMP + DTEMP
      PARARR(INDEX) = PAR  + DPAR * PAR
      GOTO 1
C
99    WRITE(*,*) ' RdEnv : Einlesefehler '
      CLOSE(25)
      STOP
C
20    CLOSE(25)
      RETURN
      END
C ====================================================================
      SUBROUTINE ActEnv(T)
C     -------------------------------------------------
C     Subroutine ActEnv: Es werden die zum Zeitpunkt
C     T gueltigen Temperatur- und PAR-Werte ermittelt
C     (lineare Interpolation zwischen den naechsten
C     Werten). Die Bodentemperatur wird als gleitendes
C     Tagesmittel bestimmt.
C     -------------------------------------------------
      REAL*8 PAR, TEMP, PRECIP, T0, T1, T, TSOIL, ACTPAR
      REAL*8 PARARR(30000), TEMPAR(30000)
      INTEGER i, j, NSOIL, INDEX
C
      COMMON /READ/   PARARR, TEMPAR
      COMMON /ENVIR/  PAR, TEMP, PRECIP, TSOIL, ACTPAR
C
      SAVE /READ/, /ENVIR/
C
      NSOIL = 0
      TSOIL = 0.0
C
      INDEX = INT(T * 4. * 24.)
C     -------------------------------------------------
C     Berechnung der Bodentemperatur als gleitendes
C     Mittel der letzten 24 Stunden
C     -------------------------------------------------
      DO 10 i=INDEX-96,INDEX
        IF (TEMPAR(i) .GT. -90.) THEN
          TSOIL = TSOIL + TEMPAR(i)
          NSOIL = NSOIL + 1
        ENDIF
 10   CONTINUE
      IF ( NSOIL .GT. 0 ) THEN
        TSOIL = TSOIL / FLOAT(NSOIL)
      ELSE
        TSOIL = TEMP
      ENDIF
C
C     -------------------------------------------------
C     Bestimmung der fuer den Zeitpunkt T naechsten,
C     gueltigen Messwerte von Temperatur und PAR.
C     Lineare Interpolation zwischen diesen Werten
C     -------------------------------------------------
      i = INDEX
 20   IF ((TEMPAR(i) .LT. -90) .OR. (PARARR(i) .LT. -90.)) THEN
        i = i - 1
        GOTO 20
      ENDIF
      j = INDEX + 1
 21   IF ((TEMPAR(j) .LT. -90) .OR. (PARARR(j) .LT. -90.)) THEN
        j = j + 1
        GOTO 21
      ENDIF
C
      T0   = FLOAT(i) / 4. / 24.
      T1   = FLOAT(j) / 4. / 24.
      TEMP = DBLE((TEMPAR(j)-TEMPAR(i))/(T1-T0)*(T-T0)+TEMPAR(i))
      PAR  = DBLE((PARARR(j)-PARARR(i))/(T1-T0)*(T-T0)+PARARR(i))
C
      RETURN
      END
C ====================================================================
      SUBROUTINE ActKPH()
C     -------------------------------------------------
C     Subroutine ActKPH : Bestimmung der fuer ACTPAR
C     gueltigen Quantenwirksamkeit der Photosynthese
C     (KPH) der einzelnen Kompartimente.
C     -------------------------------------------------
      REAL*8 PAR, TEMP, PRECIP, TSOIL, ACTPAR
      REAL*8 KPH(4),KR(4),KE(4),KL(4),KA(4), KG1, KG2, KT
      REAL*8 KPHW(200)
C
      INTEGER ipar
C
      COMMON /ENVIR/ PAR, TEMP, PRECIP, TSOIL, ACTPAR
      COMMON /KOEFF/ KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /KPHWERT/ KPHW
C
      SAVE /ENVIR/, /KOEFF/, /KPHWERT/
C
      IF (ACTPAR .LT. 0.0) THEN
        ipar   = 0
      ENDIF
      IF (ACTPAR .LT. 2000.0) THEN
        ipar   = INT(ACTPAR+0.5)/10
      ELSE
        ipar   = 200
      ENDIF
C
      KPH(1) = KPHW(ipar)
      KPH(2) = KPHW(ipar)
      KPH(3) = 0.0
      KPH(4) = KPHW(ipar)
C
      RETURN
      END
C ====================================================================
      SUBROUTINE ActKoe(T)
C     -------------------------------------------------
C     Subroutine ActKoe : Berechnung der zum Zeitpunkt
C     T gueltigen Werte der Koeffizienten KR,KA,KE,KL
C     -------------------------------------------------
      REAL*8 T, PAR, TEMP, PRECIP, TSOIL, ACTPAR
      REAL*8 KAW(4,365), KPHW(200)
      REAL*8 KPH(4),KR(4),KE(4),KL(4),KA(4), KG1, KG2, KT
      INTEGER i, j, izeit
C
      COMMON /ENVIR/  PAR, TEMP, PRECIP, TSOIL, ACTPAR
      COMMON /KOEFF/  KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /KAWERT/ KAW
C
      SAVE /ENVIR/,/KOEFF/,/KAWERT/
C
C     -------------------------------------------------
C     Berechnung der Respirationskoeffizienten fuer
C     Halme, Wurzeln und Aehren in Abhaengigkeit der
C     Temperatur. Der Koeffizient der Blaetter wird
C     aufgrund der zusaetzlich beruecksichtigten Ab-
C     haengigkeit von dem Glc-Gehalt direkt in der
C     Subroutine FCN berechnet.
C     -------------------------------------------------
      KR(2) = 0.0   + 0.00482 * EXP(0.0878 * TEMP)
      KR(3) = 0.0   + 0.01687 * EXP(0.0750 * TEMP)
      KR(4) = 0.0103+ 0.0014  * EXP(0.1085 * TEMP)
C
C     -------------------------------------------------
C     Die Extinktionskoeffizienten sind konstant
C     -------------------------------------------------
      KE(1) = 0.047
      KE(2) = 0.029
      KE(3) = 0.0
      KE(4) = 0.01
C
C     -------------------------------------------------
C     Berechnung der zum Zeitpunkt T gueltigen Alloka-
C     tionskoeffizienten
C     -------------------------------------------------
      izeit = INT(T+0.5)
C
      DO 20 j=1,4
 25     IF (KAW(1,izeit) .LT. -99.0) THEN
          izeit = izeit - 1
          GOTO 25
        ELSE
          KA(j) = KAW(j,izeit)
         IF (KA(j) .LE. 0.0) KA(j) = 0.0
        ENDIF
  20  CONTINUE
C
C     -------------------------------------------------
C     Bestandesabfallproduktion ist in dieser Modell-
C     version nicht beruecksichtigt.
C     -------------------------------------------------
      KL(1) = 0.0
      KL(2) = 0.0
      KL(3) = 0.0
      KL(4) = 0.0
C
      RETURN
      END
C ====================================================================
      SUBROUTINE TRANS(I, J)
C     -------------------------------------------------
C     Subroutine Trans: Berechnung der Translokation
C     zwischen den Kompartimenten mit den Strukturpools
C     I und J. Translokationsgewinne (TRANSP) und
C     -verluste (TRANSM) werden ueber den COMMON-Block
C     TRANSL an FCN uebergeben.
C     -------------------------------------------------
      REAL*8 TRANSM(75), TRANSP(75), Y(75)
      REAL*8 KPH(4),KR(4),KE(4),KL(4), KA(4), KG1, KG2, KT
      REAL*8 CI, CJ, BIOI, BIOJ
      INTEGER I, J
C
      COMMON /KOEFF/  KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /TRANSL/ TRANSM, TRANSP
      COMMON /POOLS/  Y
C
      SAVE
C
      BIOI = Y(I) + Y(I+2) + Y(I+4)
      BIOJ = Y(J) + Y(J+2) + Y(J+4)
C
      CI = 0.0
      CJ = 0.0
      IF (Y(I+2) .GT. 0.0 .AND. BIOI .GT. 0.0) CI = Y(I+2) / BIOI
      IF (Y(J+2) .GT. 0.0 .AND. BIOJ .GT. 0.0) CJ = Y(J+2) / BIOJ
C
      TRANSM(I+2) = TRANSM(I+2) + KT * Y(I+2) * (CI - CJ)
      TRANSP(J+2) = TRANSP(J+2) + KT * Y(I+2) * (CI - CJ)
      TRANSM(J+2) = TRANSM(J+2) + KT * Y(J+2) * (CJ - CI)
      TRANSP(I+2) = TRANSP(I+2) + KT * Y(J+2) * (CJ - CI)
C
      RETURN
      END
C ====================================================================
      SUBROUTINE FCN (T, Y, F)
C     -----------------------------------------------
C     Subroutine FCN : Integrationsroutine.
C     -----------------------------------------------
      REAL*8 Y(75), F(75), YP(75), P(4), R(4), A(4), LP(4)
      REAL*8 TRANSM(75), TRANSP(75)
      REAL*8 F1, F2, TERM1, TERM2, GLCPCT
      REAL*8 T, PAR, TEMP, PRECIP, TSOIL, ACTPAR, PARABS
      REAL*8 KPH(4),KR(4),KE(4),KL(4), KA(4), KT, KG1, KG2
      REAL*8 YLIMIT(2), G1(4),G2(4)
      REAL*8 PHOTOSUM, RESSUM, ALLOSUM, LPSUM
      REAL*8  FSTR(5), FASS(4), FGLC(4)
      REAL*8 BIO, YALL, YMINUS
C
      INTEGER i,j,k,l, MAXLAY(2), NLAYERS, ML
      integer id(75)
C
C
      COMMON /POOLS/   YP
      COMMON /TRANSL/  TRANSM, TRANSP
      COMMON /ENVIR/   PAR, TEMP, PRECIP, TSOIL, ACTPAR
      COMMON /KOEFF/   KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /GEN1/    YLIMIT, NLAYERS
      COMMON /FLUXES/  PHOTOSUM, RESSUM, ALLOSUM, LPSUM,
     &                 FSTR, FASS, FGLC
      COMMON /LAYER/   MAXLAY
      COMMON /BILANZ/  YALL, YMINUS
C
      SAVE
C
      ALLOSUM  = 0.0
      LPSUM    = 0.0
      RESSUM   = 0.0
      PHOTOSUM = 0.0
c
      do 1 i=1,4
        FSTR(i) = 0.0
        FASS(i) = 0.0
        FGLC(i) = 0.0
  1   continue
      FSTR(5) = 0.0
c
      YALL    = 0.0
      YMINUS  = 0.0
C
      CALL ActEnv(T)
      CALL ActKoe(T)
C
      DO 3 i=1,NLAYERS*6+9
        id(i) = 0
        YALL = YALL + Y(i)
        IF (Y(i) .LT. 0.0) THEN
          YMINUS = YMINUS + Y(i)
          Y(i)  = 0.0
          id(i) = 1
        ENDIF
        YP(i) = Y(i)
        TRANSP(i) = 0.0
        TRANSM(i) = 0.0
        F(i)      = 0.0
   3  CONTINUE
C
C     -------------------------------------------------
C     Bestimmung der hoechsten gefuellten Schicht
C     -------------------------------------------------
      ML = MAX(MAXLAY(1),MAXLAY(2))
C
      ACTPAR = PAR
C     -------------------------------------------------
C     Berechnung der durch die Aehren absorbierten PAR
C     und der Aehrenphotosynthese
C     -------------------------------------------------
      CALL ActKph()
      PARABS    = ACTPAR * (1. - EXP(-KE(4) *
     &          (Y(NLAYERS*6+2)+Y(NLAYERS*6+4)+Y(NLAYERS*6+6)) ))
      ACTPAR    = ACTPAR - PARABS
      P(4)      = KPH(4) * 1.E-6 * 3600. * 24.
     &                  * PARABS
      PHOTOSUM  = PHOTOSUM + P(4)
C
C     -------------------------------------------------
C     Berechnung der Terme TERM1 und TERM2 fuer die
C     3-dimensionale Respirationsfunktion der Blaetter
C     -------------------------------------------------
      F1      = (-0.0039+0.00241*EXP(0.0961*TEMP))
      F2      = (-0.0488+0.07620*EXP(0.0343*TEMP))
      TERM1   = (F2-F1)/(2.121-0.1608)
      TERM2   = F2 - (0.1608 * (F2-F1)/(2.121-0.1608))
C
C     -------------------------------------------------
C     Alle Berechnungen fuer Blaetter (i=1) und Halme
C     (i=2) aller Schichten l ausgehend von der ober-
C     sten gefuellten Schicht (l=ML) bis zur niedrig-
C     sten Schicht (l=1)
C     -------------------------------------------------
      DO 10 l=ML,1,-1
        k = (l-1)*6
C
        DO 20 i=1,2
C
          BIO = Y(k+i)+Y(k+i+2)+Y(k+i+4)
C
C         ---------------------------------------------
C         Photosynthese:
C         1. Bestimmung der akt. Quantenwirksamkeit
C         2. Berechnung der absorbierten PAR
C         3. Berechnung des Photosyntheseflusses
C         ---------------------------------------------
          CALL ActKPH()
C
          PARABS    = ACTPAR
     &              * (1. - EXP(-KE(i) * BIO ))
          ACTPAR    = ACTPAR  - PARABS
          P(i)      = KPH(i) * 1.E-6 * 3600. * 24.
     &                * PARABS
          PHOTOSUM  = PHOTOSUM + P(i)
C         ---------------------------------------------
C         Respiration:
C         Fuer Blaetter Berechnung nach 3-D-Funktion,
C         fuer Halme nur Temperaturabhaengigkeit.
C         Respiration nur moeglich, wenn Assimilat-
C         pool nicht leer.
C         ---------------------------------------------
          IF (i .eq. 1) THEN
            IF ( Y(k+i+4) .GT. 0.1 ) THEN
              GLCPCT  = Y(k+i+4) / BIO * 100.
              IF (GLCPCT .GT. 1.4) GLCPCT = 1.4
              KR(i)   = TERM1 * GLCPCT + TERM2
              R(i)    = KR(i)  * BIO
              RESSUM  = RESSUM + R(i)
            ELSE
              R(i)    = 0.0
            ENDIF
          ELSE
            IF ( Y(k+i+4) .GT. 0.1 ) THEN
              R(i)    = KR(i)  * BIO
              RESSUM  = RESSUM + R(i)
            ELSE
              R(i)    = 0.0
            ENDIF
          ENDIF
C
C         ---------------------------------------------
C         Litterproduktion:
C         ---------------------------------------------
          LP(i)     = KL(i)  * BIO
          LPSUM     = LPSUM  + LP(i)
C
C         ---------------------------------------------
C         Allokation:
C         Bei Erreichen des Schichtschwellenwertes wird
C         Allokation durch sigmoide Funktion begrenzt.
C         ---------------------------------------------
          IF (Y(k+i+2) .GT. 0.0) THEN
            A(i) = KA(i) * Y(k+i+2)
     &             * (1. - 1./(1.+EXP(-1*(BIO-YLIMIT(i)/1.))))
            ALLOSUM = ALLOSUM + A(i)
          ELSE
            A(i)    = 0.0
          ENDIF
C
C         ---------------------------------------------
C         Translokation:
C         ---------------------------------------------
          IF ( i .EQ. 2 .AND. l .GT. 1) THEN
C           -------------------------------------------
C           Translokation mit naechst niedriger Schicht
C           nur fuer Halme
C           -------------------------------------------
            CALL TRANS(k-6+i, k+i)
          ENDIF
C
          IF ( i .EQ. 1) THEN
C           -------------------------------------------
C           Translokation zwischen Blaettern und Halmen
C           der gleichen Schicht
C           -------------------------------------------
            CALL TRANS(k+i, k+i+1)
          ENDIF
C
C         ---------------------------------------------
C         Konversion Glucose->Assimilate (G1) und vice
C         versa (G2)
C         ---------------------------------------------
          G1(i) = KG1 * Y(k+i+4) * Y(k+i+4)
          G2(i) = KG2 * Y(k+i+2)
C
C         ---------------------------------------------
C         Summation der Fluesse (ohne Translokation)
C         ---------------------------------------------
          F(k+i)   = A(i)  - LP(i)
          F(k+i+2) = G1(i) - A(i) - G2(i)
          F(k+i+4) = P(i)  - R(i) - G1(i) + G2(i)
C
  20    CONTINUE
C
 10   CONTINUE
C
C     -------------------------------------------------
C     Alle Berechnungen fuer Wurzeln (i=3) und Aehren
C     (i=4).
C     -------------------------------------------------
C
      DO 30 i=3,4
        BIO = Y(NLAYERS*6+i-2) + Y(NLAYERS*6+i) + Y(NLAYERS*6+i+2)
C       -----------------------------------------------
C       Respiration :
C       -----------------------------------------------
        IF (Y(NLAYERS*6+i+2) .GT. 0.001) THEN
          R(i)    =  KR(i) * BIO
        ELSE
          R(i)    = 0.0
        ENDIF
        RESSUM   = RESSUM  + R(i)
C
C       -----------------------------------------------
C       Allokation :
C       -----------------------------------------------
        A(i)     = KA(i) * Y(NLAYERS*6+i)
        ALLOSUM  = ALLOSUM + A(i)
C
C       -----------------------------------------------
C       Konversion Glucose->Assimilate (G1) und vice
C       versa (G2)
C       -----------------------------------------------
        G1(i)    = KG1   * Y(NLAYERS*6+i+2) * Y(NLAYERS*6+i+2)
        G2(i)    = KG2   * Y(NLAYERS*6+i)
C
C       -----------------------------------------------
C       Bestandesabfallproduktion
C       -----------------------------------------------
        LP(i)    = KL(i) * BIO
        LPSUM    = LPSUM   + LP(i)
C
C       -----------------------------------------------
C       Summation der Fluesse (ohne Translokation)
C       -----------------------------------------------
        F(NLAYERS*6+i-2) =        A(i) - LP(i)
        F(NLAYERS*6+i)   =      - A(i) - G2(i) + G1(i)
        F(NLAYERS*6+i+2) = P(i) - R(i) + G2(i) - G1(i)
C
 30   CONTINUE
C
C     -------------------------------------------------
C     Translokation zwischen Wurzeln und Halmen
C     -------------------------------------------------
      CALL TRANS(2, NLAYERS*6+1)
C
C     -------------------------------------------------
C     Translokation zwischen Aehren und Halmen
C     -------------------------------------------------
      IF (Y(NLAYERS*6+4) .GT. 0.001) THEN
          CALL TRANS(MAXLAY(2)*6-6+2, NLAYERS*6+2)
      ENDIF
C
C     -------------------------------------------------
C     Allokation von den Assimilaten der Aehren in das
C     Endosperm. Keine Respiration fuer Endosperm.
C     -------------------------------------------------
      FSTR(5)        = KA(4) * Y(NLAYERS*6+4)
      F(NLAYERS*6+4) = F(NLAYERS*6+4) - FSTR(5)
      F(NLAYERS*6+7) = FSTR(5)
C
C     -------------------------------------------------
C     Addition der Translokation zu den Fluessen fuer
C     Assimilate von Blaettern und Halmen
C     -------------------------------------------------
      DO 40 i=0,NLAYERS-1
        DO 42 j=1,2
          F(i*6+j+2) = F(i*6+j+2) + TRANSP(i*6+j+2)
     &                            - TRANSM(i*6+j+2)
          FSTR(j)    = FSTR(j) + F(i*6+j)
          FASS(j)    = FASS(j) + F(i*6+j+2)
          FGLC(j)    = FGLC(j) + F(i*6+j+4)
  42    CONTINUE
  40  CONTINUE
C
C     -------------------------------------------------
C     Addition der Translokation zu den Fluessen fuer
C     Assimilate von Wurzeln und Aehren
C     -------------------------------------------------
      DO 50 i=3,4
        F(NLAYERS*6+i) = F(NLAYERS*6+i) + TRANSP(NLAYERS*6+i)
     &                                  - TRANSM(NLAYERS*6+i)
        FSTR(i) = F(NLAYERS*6+i-2)
        FASS(i) = F(NLAYERS*6+i)
        FGLC(i) = F(NLAYERS*6+i+2)
  50  CONTINUE
C
C     -------------------------------------------------
C     Berechnung der Fluesse fuer die Hilfspools Atmos-
C     phaere und Bestandesabfall
C     -------------------------------------------------
      F(NLAYERS*6+8) = F(NLAYERS*6+8) + RESSUM - PHOTOSUM
      F(NLAYERS*6+9) = F(NLAYERS*6+9) + LPSUM
C
      RETURN
      END
C ====================================================================
      SUBROUTINE OUTPUT(XSOL, Y)
C     -------------------------------------------------
C     Subroutine Output : Ausgabe
C     -------------------------------------------------
      REAL*8  XSOL, Y(75), YLIMIT(2)
      REAL*8 KPH(4),KR(4),KE(4),KL(4), KA(4), KT, KG1, KG2
      REAL*8  ALLOSUM, LPSUM, RESSUM,PHOTOSUM, NPP
      REAL*8  YSUM,STR(5), ASS(4), GLC(4), ALL(4)
      REAL*8  PCTGLC(4), PCTASS(4)
      REAL*8  BALL(11),HALL(11),WALL,ENALL, YALL, YMINUS
      REAL*8  ALLSTR, ALLASS, ALLGLC, ALLPCT
      REAL*8  FSTR(5), FASS(4), FGLC(4)
      REAL*8  TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      INTEGER i, j, NLAYERS, MAXLAY(2)
      INTEGER IGLC(4), IASS(4)
C
      COMMON /ZEIT/    TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      COMMON /KOEFF/   KPH, KE, KR, KL, KA, KG1, KG2, KT
      COMMON /GEN1/    YLIMIT, NLAYERS
      COMMON /FLUXES/  PHOTOSUM, RESSUM, ALLOSUM, LPSUM,
     &                 FSTR, FASS, FGLC
      COMMON /LAYER/   MAXLAY
      COMMON /BILANZ/  YALL, YMINUS
C
      common /dd/      STR,ASS,GLC,ALL,YSUM,ALLSTR,ALLASS,ALLGLC
C
      SAVE
C
      i=0
      j=0
C
      DO 1 j=1,4
        STR(j) = 0.0
        ASS(j) = 0.0
        GLC(j) = 0.0
        ALL(j) = 0.0
   1  CONTINUE
C
      YSUM   = 0.0
      DO 3 j=1,NLAYERS*6+9
        YSUM = YSUM + Y(j)
   3  CONTINUE
C
      DO 5 i=0,NLAYERS-1
        DO 7 j=1,2
          STR(j) = STR(j) + Y(i*6+j)
          ASS(j) = ASS(j) + Y(i*6+j+2)
          GLC(j) = GLC(j) + Y(i*6+j+4)
    7   CONTINUE
  5   CONTINUE
      DO 9 j=3,4
        STR(j)   = Y(NLAYERS*6+j-2)
        ASS(j)   = Y(NLAYERS*6+j)
        GLC(j)   = Y(NLAYERS*6+j+2)
  9   CONTINUE
      STR(5)     = Y(NLAYERS*6+7)
C
      ALLSTR = STR(1) + STR(2) + STR(3) + STR(4) + STR(5)
      ALLASS = ASS(1) + ASS(2) + ASS(3) + ASS(4)
      ALLGLC = GLC(1) + GLC(2) + GLC(3) + GLC(4)
C
C     -------------------------------------------------
C     fuer jedes Kompartiment berechnen :
C     gesamte Pools an Struktursubstanz, Assimilate u.
C     Glucose [gC/m^2] sowie Assimilat- und Glucosege-
C     halte [%]
C     -------------------------------------------------
      DO 15 i=1,4
        PCTGLC(i) = 0.0
        PCTASS(i) = 0.0
        IF (i .NE. 4) THEN
          ALL(i) = STR(i)+ASS(i)+GLC(i)
        ELSE
          ALL(i) = STR(i)+ASS(i)+GLC(i)+STR(5)
        ENDIF
        IF (ALL(i) .GT. 0.0) PCTGLC(i) = GLC(i)/ALL(i)*100.
        IF (ALL(i) .GT. 0.0) PCTASS(i) = ASS(i)/ALL(i)*100.
  15  CONTINUE
      ALLPCT = (ALLASS+ALLGLC) / (ALLSTR+ALLASS+ALLGLC)*100.
C
C
      IF (ABS(XSOL-TNEXT2) .LT. 0.01) THEN
C       -----------------------------------------------
C       Dateiausgabe (nicht schichtenaufgeloest) :
C         Zeit [d],
C         Gesamt, Struktur, Assimilate, Glucose
C           (jeweils fuer Gesamtphytomasse,
C            Blaetter (1), Halme (2), Wurzeln (3),
C            Aehren (4) in [gC/m^2] ),
C         Endosperm [gC/m^2]
C       -----------------------------------------------
        WRITE(30,998)XSOL,
     &               YSUM, ALLSTR, ALLASS, ALLGLC,
     &               ALL(1),STR(1),ASS(1),GLC(1),
     &               ALL(2),STR(2),ASS(2),GLC(2),
     &               ALL(3),STR(3),ASS(3),GLC(3),
     &               ALL(4),STR(4),ASS(4),GLC(4),
     &               STR(5)
C
C       -----------------------------------------------
C       Dateiausgabe (nicht schichtenaufgeloest) :
C         Zeit,
C         Assimilatgehalt, Glucosegehalt [%]
C           ( jeweils fuer Gesamtphytomasse,
C             Blaetter (1), Halme (2),
C             Wurzeln (3), Aehren(4) )
C       -----------------------------------------------
        WRITE(31,997) XSOL,
     &                ALLASS/YSUM*100., ALLGLC/YSUM*100.,
     &                PCTASS(1),PCTGLC(1),
     &                PCTASS(2),PCTGLC(2),
     &                PCTASS(3),PCTGLC(3),
     &                PCTASS(4),PCTGLC(4)
C
        TNEXT2 = TNEXT2 + TFILE
C
      ENDIF
C
      IF (ABS(XSOL-TNEXT1) .LT. 0.01) THEN
C       -----------------------------------------------
C       Bildschirmausgabe (nicht schichtenaufgeloest) :
C       Zeit [d],
C       hoechste gefuellte Schicht,
C       Phytomasse [gC/m^2], Assimilat- und Glucosege-
C       gehalt [%] (jeweils fuer gesamten Bestand,
C       Blaetter (j=1), Halme (j=2), Wurzeln (j=3) und
C       Aehren (j=4)).
C       -----------------------------------------------
        WRITE(*,996) XSOL,MAXLAY(1),
     &  YSUM, INT( ALLASS/YSUM * 100. +0.5),
     &        INT( ALLGLC/YSUM * 100. +0.5),
     &  (ALL(j), INT(PCTASS(j)+0.5), INT(PCTGLC(j)+0.5), j=1,4)
C
C       -----------------------------------------------
C       Dateiausgabe (schichtenaufgeloest) :
C       Zeit [d],
C       Gesamtphytomasse Wurzeln [gC/m^2],
C       Gesamtphytomasse Blaetter u. Halme (jeweils von
C       unterster bis oberster Schicht) [gC/m^2],
C       Gesamtphytomasse Aehren, Endosperm [gC/m^2]
C       -----------------------------------------------
        WRITE(40,449) XSOL,
     &                (Y(NLAYERS*6+5)+Y(NLAYERS*6+3)+Y(NLAYERS*6+1)),
     &                ((Y(i*6+5)+Y(i*6+3)+Y(i*6+1)),
     &                 (Y(i*6+6)+Y(i*6+4)+Y(i*6+2)), i=0,NLAYERS-1),
     &                (Y(NLAYERS*6+6)+Y(NLAYERS*6+4)+Y(NLAYERS*6+2)),
     &                Y(NLAYERS*6+7)
C
C       -----------------------------------------------
C       Dateiausgabe (schichtenaufgeloest) :
C       Zeit [d],
C       Pools Glucose, Assimilate, Struktursubstanz
C       (jeweils f. Wurzeln,
C        Blaetter u. Halme (je von Schicht 1 bis NLAYERS),
C        und Aehren),
C       Pool Endosperm  ( alles in [gC/m^2])
C       ------------------------------------------------
        WRITE(41,449) XSOL,
     &                Y(NLAYERS*6+5),Y(NLAYERS*6+3),Y(NLAYERS*6+1),
     &                ( Y(i*6+5),Y(i*6+3),Y(i*6+1),
     &                  Y(i*6+6),Y(i*6+4),Y(i*6+2), i=0,NLAYERS-1),
     &                Y(NLAYERS*6+6),Y(NLAYERS*6+4),Y(NLAYERS*6+2),
     &                Y(NLAYERS*6+7)
C
        TNEXT1 = TNEXT1 + TSCRE
      ENDIF
C
  999 FORMAT(1F7.2, 20F10.2)
  998 FORMAT(1F7.2, 7(4F9.2,2X))
  997 FORMAT(1F7.2,2X, 5(2F7.2,2X))
  996 FORMAT(1F5.0,1X,1I2,1X,5(1F5.0,1X,1I3,1I3,2X))
  449 FORMAT(1F6.1,1X,20(3F6.1,1X))
C
      IF (TNEXT1 .LE. TNEXT2) XSOL = XSOL + TSCRE
      IF (TNEXT2 .LT. TNEXT1) XSOL = XSOL + TFILE
C
      RETURN
      END
C ====================================================================
      SUBROUTINE IniOut()
C     -------------------------------------------------
C     Subroutine IniOut : Initialisierung der Ausgabe
C     -------------------------------------------------
      REAL*8  TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      CHARACTER*40 KAPTS, KPHPTS, ENVIN,
     &             POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      COMMON /ZEIT/  TANF, TEND, TNEXT1, TNEXT2, TSCRE, TFILE
      COMMON /FILES/  KAPTS, KPHPTS, ENVIN,
     &                POUT1, POUT2, POUT3, FOUT1, FOUT2, INI
C
      SAVE
C
      TNEXT1 = TANF
      TNEXT2 = TANF
C
      OPEN(30,FILE=POUT1)
      OPEN(31,FILE=POUT2)
      OPEN(32,FILE=POUT3)
      OPEN(33,FILE=FOUT1)
      OPEN(34,FILE=FOUT2)
C
      OPEN(40,FILE='SOUT1')
      OPEN(41,FILE='SOUT2')
C
      WRITE(*,*)'     Layer      Pools (jeweils Phyto, %Assi, %Glc)'
      WRITE(*,*)' Zeit  L l  gesamt     l  Blatt      l    Halm  ',
     &          '   l   Wurzel   l   Aehre'
      WRITE(*,*)'--------------------------------------------------',
     &          '--------------------------'
c
      RETURN
      END
C ====================================================================
      SUBROUTINE EndOut()
C     -------------------------------------------------
C     Subroutine EndOut: Ausgabe beenden
C     -------------------------------------------------
C
      CLOSE(10)
      CLOSE(11)
      CLOSE(12)
C
      CLOSE(20)
      CLOSE(21)
      CLOSE(22)
      CLOSE(23)
      CLOSE(24)
      CLOSE(25)
C
      CLOSE(30)
      CLOSE(31)
      CLOSE(32)
      CLOSE(33)
      CLOSE(34)
C
      CLOSE(40)
      CLOSE(41)
C
      RETURN
      END

\end{verbatim}
